Package: VOSONDash
Version: 0.5.7
Title: User Interface for Collecting and Analysing Social Networks
Description: A 'Shiny' application for the interactive visualisation and
    analysis of networks that also provides a web interface for collecting
    social media data using 'vosonSML'.
Type: Package
Imports: data.table, graphics, httpuv, httr, igraph (>= 1.2.2),
        lattice, magrittr, RColorBrewer, shiny (>= 1.3.2), SnowballC,
        systemfonts, syuzhet, textutils, tm, utils, vosonSML (>=
        0.29.0), wordcloud
Suggests: dplyr, DT, htmlwidgets, rtweet (>= 0.6.8), shinydashboard,
        shinyjs, visNetwork
Depends: R (>= 3.2.0)
Encoding: UTF-8
Author: Bryan Gertzel, Robert Ackland
Maintainer: Bryan Gertzel <bryan.gertzel@anu.edu.au>
License: GPL (>= 3)
RoxygenNote: 7.1.1
NeedsCompilation: no
URL: https://github.com/vosonlab/VOSONDash
BugReports: https://github.com/vosonlab/VOSONDash/issues
Packaged: 2020-07-27 12:10:40 UTC; viole
Repository: CRAN
Date/Publication: 2020-07-27 13:20:02 UTC
Built: R 4.6.0; ; 2026-04-25 13:54:21 UTC; unix
