# Generated by roxygen2: do not edit by hand

export(addAdditionalMeasures)
export(applyCategoricalFilters)
export(applyComponentFilter)
export(applyGraphFilters)
export(applyPruneFilter)
export(collectRedditData)
export(collectTwitterData)
export(collectYoutubeData)
export(corpusFromGraph)
export(createRedditActorNetwork)
export(createRedditRequestUrl)
export(createTokenId)
export(createTwitterActorNetwork)
export(createTwitterDevToken)
export(createTwitterWebToken)
export(createYoutubeNetwork)
export(dtbGraph)
export(eawGraph)
export(emptyPlotMessage)
export(getNetworkMetrics)
export(getRedditUrlSubreddit)
export(getRedditUrlThreadId)
export(getSystemFontFamilies)
export(getVOSONDashVer)
export(getVertexCategories)
export(getVosonSMLVersion)
export(getYoutubeVideoId)
export(hasVosonTextData)
export(isMac)
export(isNullOrEmpty)
export(isVosonSML0290)
export(loadPackageGraph)
export(logMessage)
export(mixmat)
export(runVOSONDash)
export(systemTimeFilename)
export(wordCloudPlot)
export(wordFreqChart)
export(wordFreqFromCorpus)
export(wordSentChart)
export(wordSentData)
export(wordSentValenceChart)
import(SnowballC)
import(httpuv)
import(shiny)
importFrom(RColorBrewer,brewer.pal)
importFrom(data.table,data.table)
importFrom(graphics,barplot)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(httr,parse_url)
importFrom(igraph,'E<-')
importFrom(igraph,'V<-')
importFrom(igraph,E)
importFrom(igraph,V)
importFrom(igraph,as_edgelist)
importFrom(igraph,betweenness)
importFrom(igraph,centr_betw)
importFrom(igraph,centr_clo)
importFrom(igraph,centr_degree)
importFrom(igraph,closeness)
importFrom(igraph,components)
importFrom(igraph,count_components)
importFrom(igraph,degree)
importFrom(igraph,delete_vertices)
importFrom(igraph,ecount)
importFrom(igraph,edge_attr_names)
importFrom(igraph,graph.density)
importFrom(igraph,gsize)
importFrom(igraph,is.directed)
importFrom(igraph,is.igraph)
importFrom(igraph,mean_distance)
importFrom(igraph,reciprocity)
importFrom(igraph,set_graph_attr)
importFrom(igraph,simplify)
importFrom(igraph,transitivity)
importFrom(igraph,vcount)
importFrom(igraph,vertex_attr)
importFrom(igraph,vertex_attr_names)
importFrom(lattice,barchart)
importFrom(magrittr,'%<>%')
importFrom(magrittr,'%>%')
importFrom(systemfonts,system_fonts)
importFrom(syuzhet,get_nrc_sentiment)
importFrom(textutils,HTMLdecode)
importFrom(tm,DocumentTermMatrix)
importFrom(tm,VCorpus)
importFrom(tm,VectorSource)
importFrom(tm,content_transformer)
importFrom(tm,removeNumbers)
importFrom(tm,removePunctuation)
importFrom(tm,removeSparseTerms)
importFrom(tm,stemDocument)
importFrom(tm,stopwords)
importFrom(tm,tm_map)
importFrom(utils,packageVersion)
importFrom(vosonSML,Authenticate)
importFrom(vosonSML,Collect)
importFrom(vosonSML,Create)
importFrom(wordcloud,wordcloud)
